package blusunrize.immersiveengineering.common.items;

import blusunrize.immersiveengineering.api.Lib;
import blusunrize.immersiveengineering.api.crafting.BlueprintCraftingRecipe;
import blusunrize.immersiveengineering.common.gui.ContainerModWorkbench;
import blusunrize.immersiveengineering.common.gui.IESlot;
import blusunrize.immersiveengineering.common.util.ItemNBTHelper;
import blusunrize.immersiveengineering.common.util.Utils;
import net.minecraft.client.resources.I18n;
import net.minecraft.client.util.ITooltipFlag;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.inventory.Container;
import net.minecraft.inventory.IInventory;
import net.minecraft.inventory.Slot;
import net.minecraft.item.ItemStack;
import net.minecraft.util.NonNullList;
import net.minecraft.world.World;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;
import org.lwjgl.input.Keyboard;

import javax.annotation.Nullable;
import java.util.LinkedHashSet;
import java.util.List;

public class ItemEngineersBlueprint extends ItemUpgradeableTool
{
	public ItemEngineersBlueprint()
	{
		super("blueprint", 1, null);
	}

	@Override
	public String func_77667_c(ItemStack stack)
	{
		return this.func_77658_a();
	}

	@Override
	@SideOnly(Side.CLIENT)
	public void func_77624_a(ItemStack stack, @Nullable World world, List<String> list, ITooltipFlag flag)
	{
		String key = ItemNBTHelper.getString(stack,"blueprint");
		if(key != null && !key.isEmpty() && BlueprintCraftingRecipe.blueprintCategories.contains(key))
		{
			String formatKey = Lib.DESC_INFO + "blueprint." + key;
			String formatted = I18n.func_135052_a(formatKey);
			list.add(formatKey.equals(formatted)?key:formatted);
			if(Keyboard.isKeyDown(Keyboard.KEY_LSHIFT) || Keyboard.isKeyDown(Keyboard.KEY_RSHIFT))
			{
				list.add(I18n.func_135052_a(Lib.DESC_INFO + "blueprint.creates1"));
				BlueprintCraftingRecipe[] recipes = BlueprintCraftingRecipe.findRecipes(key);
				if(recipes.length > 0)
					for(int i = 0; i < recipes.length; i++)
						list.add(" " + recipes[i].output.func_82833_r());
			} else
				list.add(I18n.func_135052_a(Lib.DESC_INFO + "blueprint.creates0"));
		}
	}


	@Override
	@SideOnly(Side.CLIENT)
	public void func_150895_a(CreativeTabs tab, NonNullList<ItemStack> list)
	{
		if(this.func_194125_a(tab))
			for(String key : BlueprintCraftingRecipe.blueprintCategories)
			{
				ItemStack stack = new ItemStack(this);
				ItemNBTHelper.setString(stack, "blueprint", key);
				list.add(stack);
			}
	}

	@Override
	public boolean canModify(ItemStack stack)
	{
		return true;
	}
	@Override
	public Slot[] getWorkbenchSlots(Container container, ItemStack stack, IInventory invItem)
	{
		LinkedHashSet<Slot> slots = new LinkedHashSet<Slot>();

		slots.add(new IESlot.BlueprintInput(container, invItem, 0, 74, 21, stack));
		slots.add(new IESlot.BlueprintInput(container, invItem, 1, 92, 21, stack));
		slots.add(new IESlot.BlueprintInput(container, invItem, 2, 74, 39, stack));
		slots.add(new IESlot.BlueprintInput(container, invItem, 3, 92, 39, stack));
		slots.add(new IESlot.BlueprintInput(container, invItem, 4, 74, 57, stack));
		slots.add(new IESlot.BlueprintInput(container, invItem, 5, 92, 57, stack));

		BlueprintCraftingRecipe[] recipes = BlueprintCraftingRecipe.findRecipes(ItemNBTHelper.getString(stack,"blueprint"));
		for(int i=0; i<recipes.length; i++)
		{
			int y = 21 + (i < 9 ? i / 3 : (-(i - 6) / 3)) * 18;
			slots.add(new IESlot.BlueprintOutput(container, invItem, 6 + i, 118 + (i % 3 * 18), y, stack, recipes[i]));
		}
		return slots.toArray(new Slot[slots.size()]);
	}

	public void updateOutputs(ItemStack stack)
	{
		BlueprintCraftingRecipe[] recipes = BlueprintCraftingRecipe.findRecipes(ItemNBTHelper.getString(stack,"blueprint"));
		NonNullList<ItemStack> stored = this.getContainedItems(stack);
		NonNullList<ItemStack> query = NonNullList.func_191197_a(6, ItemStack.field_190927_a);
		for(int i=0; i<stored.size(); i++)
			if(i<6)
				query.set(i, stored.get(i));
			else
			{
				stored.set(i, ItemStack.field_190927_a);
				int craftable = recipes[i-6].getMaxCrafted(query);
				if(craftable>0)
					stored.set(i, Utils.copyStackWithAmount(recipes[i-6].output, Math.min(recipes[i-6].output.func_190916_E() * craftable, 64)));
			}
		this.setContainedItems(stack, stored);
	}

	public void reduceInputs(BlueprintCraftingRecipe recipe, ItemStack stack, ItemStack crafted, Container contained)
	{
		NonNullList<ItemStack> stored = this.getContainedItems(stack);
		NonNullList<ItemStack> query = NonNullList.func_191197_a(6, ItemStack.field_190927_a);
		for(int i=0; i<6; i++)
			query.set(i, stored.get(i));
		recipe.consumeInputs(query, crafted.func_190916_E()/recipe.output.func_190916_E());
		for(int i=0; i<6; i++)
			stored.set(i, query.get(i));
		this.setContainedItems(stack, stored);
		if (contained instanceof ContainerModWorkbench)
		{
			ContainerModWorkbench work = (ContainerModWorkbench) contained;
			if(work.toolInv!=null)
			{
				work.toolInv.stackList = query;
				work.func_75130_a(work.toolInv);
			}
		}
	}

	@Override
	public int getInternalSlots(ItemStack stack)
	{
		return 6 + BlueprintCraftingRecipe.findRecipes(ItemNBTHelper.getString(stack, "blueprint")).length;
	}

	@Override
	public boolean canTakeFromWorkbench(ItemStack stack)
	{
		NonNullList<ItemStack> stored = this.getContainedItems(stack);
		for(int i=0; i<6; i++)
			if(!stored.get(i).func_190926_b())
				return false;
		return true;
	}
}
